
#define WIN32_LEAN_AND_MEAN

// includes
#include <windows.h>
#include <sqlext.h>
#include "QueryResult.h"

/*------------------------------------
	Constructor
------------------------------------*/
QueryResult::QueryResult( SQLHSTMT handle )
{

	// store handle
	statement = handle;

	// fetch first result so they don't have to movefirst to start.
	SQLRETURN return_code = SQLFetch( statement );

	// set eof
	eof = return_code == SQL_NO_DATA;

}


/*------------------------------------
	Destructor
------------------------------------*/
QueryResult::~QueryResult( )
{

	// cleanup
	if( statement )
	{

		// free
		SQLFreeHandle( SQL_HANDLE_STMT, statement );
		statement = NULL;

	}

}

/*------------------------------------
	ReleaseStatement
------------------------------------*/
void QueryResult::ReleaseStatement( )
{

	// cleanup
	if( statement )
	{

		// free
		SQLFreeHandle( SQL_HANDLE_STMT, statement );
		statement = NULL;

	}

}

/*------------------------------------
	IsEOF
------------------------------------*/
bool QueryResult::IsEOF( )
{

	return eof;

}


/*------------------------------------
	MoveNext
------------------------------------*/
void QueryResult::MoveNext( )
{

	// get the next result in the dataset
	SQLRETURN return_code = SQLFetchScroll( statement, SQL_FETCH_NEXT, 0 );

	// set eof
	eof = return_code == SQL_NO_DATA;

}

/*------------------------------------
	MovePrevious
------------------------------------*/
void QueryResult::MovePrevious( )
{

	// get the previous result in the dataset
	SQLRETURN return_code = SQLFetchScroll( statement, SQL_FETCH_PREV, 0 );

	// set eof
	eof = return_code == SQL_NO_DATA;

}

/*------------------------------------
	MoveFirst
------------------------------------*/
void QueryResult::MoveFirst( )
{

	// get the first result in the dataset
	SQLRETURN return_code = SQLFetchScroll( statement, SQL_FETCH_FIRST, 0 );

	// set eof
	eof = return_code == SQL_NO_DATA;

}

/*------------------------------------
	MoveLast
------------------------------------*/
void QueryResult::MoveLast( )
{

	// get the last result in the dataset
	SQLRETURN return_code = SQLFetchScroll( statement, SQL_FETCH_LAST, 0 );

	// set eof
	eof = return_code == SQL_NO_DATA;

}

/*------------------------------------
	GetData
------------------------------------*/
bool QueryResult::GetData( SQLUSMALLINT column, SQLSMALLINT type, SQLPOINTER value, SQLLEN length )
{

	SQLINTEGER out_length;

	// get the data
	SQLGetData( statement, column, type, value, length, &out_length );

	// got data?
	return out_length != SQL_NULL_DATA;

}

/*------------------------------------
	GetColumnInfo
------------------------------------*/
void QueryResult::GetColumnInfo( SQLUSMALLINT column, SQLSMALLINT* type, SQLUINTEGER* length )
{

	// get
	SQLDescribeCol( statement, column, NULL, 0, NULL, type, length, NULL, NULL );

}

/*------------------------------------
	GetColumnIndex
------------------------------------*/
SQLUSMALLINT QueryResult::GetColumnIndex( const char* name )
{

	char buffer[ 32 ];
	SQLSMALLINT name_length;

	// get the number of columns we have
	SQLSMALLINT columns;
	SQLNumResultCols( statement, &columns );

	// loop through all the columns until we find one that matches the name
	int index = 1;
	while( index < columns )
	{

		// get name
		ZeroMemory( buffer, 32 );
		SQLDescribeCol(
			statement,
			index,
			(SQLCHAR*)buffer,
			32,
			&name_length,
			NULL,
			NULL,
			NULL,
			NULL

		);

		// match?
		if( strcmp( buffer, name ) == 0 )
		{

			return index;

		}

		// index
		index++;

	}

	return 0;

}
