
#define WIN32_LEAN_AND_MEAN

// includes
#include <windows.h>
#include <sqlext.h>
#include "../../gmodinterface/GMLuaModule.h"
#include "DatabaseConnection.h"
#include "QueryResult.h"

//---------- Setup module
GMOD_MODULE( Init, Shutdown );

//---------- Metatypes
enum MetaTypes
{

	TYPE_DATABASE_CONNECTION = 1751,
	TYPE_DATABASE_RESULT,

};

/*------------------------------------
	GCDatabaseConnection
------------------------------------*/
LUA_FUNCTION( GCDatabaseConnection )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_CONNECTION );

	// get connection
	DatabaseConnection* database = (DatabaseConnection*)Lua()->GetUserData( 1 );
	if( database )
	{

		delete database;

	}

	return 0;

}

/*------------------------------------
	IsConnected
------------------------------------*/
LUA_FUNCTION( IsConnected )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_CONNECTION );

	// get connection
	DatabaseConnection* database = (DatabaseConnection*)Lua()->GetUserData( 1 );
	if( database )
	{

		// push the connected flag
		Lua()->Push( database->IsConnected() );
		return 1;

	}

	return 0;

}

/*------------------------------------
	SetTimeouts
------------------------------------*/
LUA_FUNCTION( SetTimeouts )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_CONNECTION );
	Lua()->CheckType( 2, GLua::TYPE_NUMBER );
	Lua()->CheckType( 3, GLua::TYPE_NUMBER );

	// get connection
	DatabaseConnection* database = (DatabaseConnection*)Lua()->GetUserData( 1 );
	if( database )
	{

		// call
		database->SetTimeouts( Lua()->GetInteger( 2 ), Lua()->GetInteger( 3 ) );

	}

	return 0;

}

/*------------------------------------
	Connect
------------------------------------*/
LUA_FUNCTION( Connect )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_CONNECTION );
	Lua()->CheckType( 2, GLua::TYPE_STRING );

	// get connection
	DatabaseConnection* database = (DatabaseConnection*)Lua()->GetUserData( 1 );
	if( database )
	{

		// call
		Lua()->Push( database->Connect( Lua()->GetString( 2 ) ) );
		return 1;

	}

	return 0;

}

/*------------------------------------
	Disconnect
------------------------------------*/
LUA_FUNCTION( Disconnect )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_CONNECTION );

	// get connection
	DatabaseConnection* database = (DatabaseConnection*)Lua()->GetUserData( 1 );
	if( database )
	{

		// call
		database->Disconnect();

	}

	return 0;

}

/*------------------------------------
	ExecuteNoResult
------------------------------------*/
LUA_FUNCTION( ExecuteNoResult )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_CONNECTION );
	Lua()->CheckType( 2, GLua::TYPE_STRING );

	// get connection
	DatabaseConnection* database = (DatabaseConnection*)Lua()->GetUserData( 1 );
	if( database )
	{

		// call
		Lua()->Push( (float)database->ExecuteNoResult( Lua()->GetString( 2 ) ) );
		return 1;

	}

	return 0;

}

/*------------------------------------
	Execute
------------------------------------*/
LUA_FUNCTION( Execute )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_CONNECTION );
	Lua()->CheckType( 2, GLua::TYPE_STRING );

	// get connection
	DatabaseConnection* database = (DatabaseConnection*)Lua()->GetUserData( 1 );
	if( database )
	{

		// get result
		QueryResult* query = database->Execute( Lua()->GetString( 2 ) );
		if( query )
		{

			// push
			ILuaObject* meta = Lua()->GetMetaTable( "QueryResult", TYPE_DATABASE_RESULT );
			Lua()->PushUserData( meta, query );
			return 1;

		}

	}

	return 0;

}

/*------------------------------------
	LastError
------------------------------------*/
LUA_FUNCTION( LastError )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_CONNECTION );

	// get connection
	DatabaseConnection* database = (DatabaseConnection*)Lua()->GetUserData( 1 );
	if( database )
	{

		// push last error
		Lua()->Push( database->LastError() );
		return 1;

	}

	return 0;

}

/*------------------------------------
	GCQueryResult
------------------------------------*/
LUA_FUNCTION( GCQueryResult )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		delete query;

	}

	return 0;

}

/*------------------------------------
	IsEOF
------------------------------------*/
LUA_FUNCTION( IsEOF )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		// push eof
		Lua()->Push( query->IsEOF() );
		return 1;

	}

	return 0;

}

/*------------------------------------
	MoveNext
------------------------------------*/
LUA_FUNCTION( MoveNext )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		// move
		query->MoveNext();

	}

	return 0;

}

/*------------------------------------
	MovePrevious
------------------------------------*/
LUA_FUNCTION( MovePrevious )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		// move
		query->MovePrevious();

	}

	return 0;

}


/*------------------------------------
	MoveFirst
------------------------------------*/
LUA_FUNCTION( MoveFirst )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		// move
		query->MoveFirst();

	}

	return 0;

}

/*------------------------------------
	MoveLast
------------------------------------*/
LUA_FUNCTION( MoveLast )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		// move
		query->MoveLast();

	}

	return 0;

}

/*------------------------------------
	ReleaseStatement
------------------------------------*/
LUA_FUNCTION( ReleaseStatement )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		// move
		query->ReleaseStatement();

	}

	return 0;

}

/*------------------------------------
	GetColumnID
------------------------------------*/
LUA_FUNCTION( GetColumnID )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );
	Lua()->CheckType( 2, GLua::TYPE_STRING );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		// push the column name
		Lua()->Push( (float)query->GetColumnIndex( Lua()->GetString( 2 ) ) );
		return 1;

	}

	return 0;

}

/*------------------------------------
	GetString
------------------------------------*/
LUA_FUNCTION( GetString )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );
	Lua()->CheckType( 2, GLua::TYPE_NUMBER );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		int index = Lua()->GetInteger( 2 );

		// get the field type and length
		SQLSMALLINT type;
		SQLUINTEGER length;
		query->GetColumnInfo( index, &type, &length );

		// allocate space for it
		SQLCHAR* buffer = new SQLCHAR[ length ];
		ZeroMemory( buffer, length );

		// fetch
		if( query->GetData( index, SQL_C_CHAR, buffer, length ) )
		{

			// push string
			Lua()->Push( (const char*)buffer );

		}
		else
		{

			// push nil
			Lua()->PushNil();

		}

		// cleanup
		delete [] buffer;

		return 1;

	}

	return 0;

}

/*------------------------------------
	GetInteger
------------------------------------*/
LUA_FUNCTION( GetInteger )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );
	Lua()->CheckType( 2, GLua::TYPE_NUMBER );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		int index = Lua()->GetInteger( 2 );

		// variable to hold the integer
		SQLINTEGER value;

		// fetch
		if( query->GetData( index, SQL_C_LONG, &value, 0 ) )
		{

			// push string
			Lua()->Push( (float)value );
			return 1;

		}

	}

	return 0;

}

/*------------------------------------
	GetFloat
------------------------------------*/
LUA_FUNCTION( GetFloat )
{

	// check
	Lua()->CheckType( 1, TYPE_DATABASE_RESULT );
	Lua()->CheckType( 2, GLua::TYPE_NUMBER );

	// get result
	QueryResult* query = (QueryResult*)Lua()->GetUserData( 1 );
	if( query )
	{

		int index = Lua()->GetInteger( 2 );

		// variable to hold the float
		SQLREAL value;

		// fetch
		if( query->GetData( index, SQL_C_FLOAT, &value, 0 ) )
		{

			// push string
			Lua()->Push( value );
			return 1;

		}

	}

	return 0;

}

/*------------------------------------
	NewDatabaseConnection
------------------------------------*/
LUA_FUNCTION( NewDatabaseConnection )
{

	// create
	DatabaseConnection* database = new DatabaseConnection();

	// push
	ILuaObject* meta = Lua()->GetMetaTable( "DatabaseConnection", TYPE_DATABASE_CONNECTION );
	Lua()->PushUserData( meta, database );

	return 1;

}


/*------------------------------------
	Init
------------------------------------*/
int Init( lua_State* L )
{

	// setup database connection metatable
	ILuaObject* MetaDatabaseConnection = Lua()->GetMetaTable( "DatabaseConnection", TYPE_DATABASE_CONNECTION );
	if( MetaDatabaseConnection )
	{

		// gc
		MetaDatabaseConnection->SetMember( "__gc", GCDatabaseConnection );

		// members
		ILuaObject* MembersDatabaseConnection = Lua()->GetNewTable();
		if( MembersDatabaseConnection )
		{

			// set members
			MembersDatabaseConnection->SetMember( "IsConnected", IsConnected );
			MembersDatabaseConnection->SetMember( "SetTimeouts", SetTimeouts );
			MembersDatabaseConnection->SetMember( "Connect", Connect );
			MembersDatabaseConnection->SetMember( "Disconnect", Disconnect );
			MembersDatabaseConnection->SetMember( "QueryNoResult", ExecuteNoResult );
			MembersDatabaseConnection->SetMember( "Query", Execute );
			MembersDatabaseConnection->SetMember( "LastError", LastError );

			// set __index
			MetaDatabaseConnection->SetMember( "__index", MembersDatabaseConnection );

		}

	}

	// setup database connection metatable
	ILuaObject* MetaQueryResult = Lua()->GetMetaTable( "QueryResult", TYPE_DATABASE_RESULT );
	if( MetaQueryResult )
	{

		// gc
		MetaQueryResult->SetMember( "__gc", GCQueryResult );

		// members
		ILuaObject* MembersQueryResult = Lua()->GetNewTable();
		if( MembersQueryResult )
		{

			// set members
			MembersQueryResult->SetMember( "IsEOF", IsEOF );
			MembersQueryResult->SetMember( "MoveFirst", MoveFirst );
			MembersQueryResult->SetMember( "MoveLast", MoveLast );
			MembersQueryResult->SetMember( "MoveNext", MoveNext );
			MembersQueryResult->SetMember( "MovePrevious", MovePrevious );
			MembersQueryResult->SetMember( "GetColumnID", GetColumnID );
			MembersQueryResult->SetMember( "Release", ReleaseStatement );
			MembersQueryResult->SetMember( "GetString", GetString );
			MembersQueryResult->SetMember( "GetInteger", GetInteger );
			MembersQueryResult->SetMember( "GetFloat", GetFloat );

			// set __index
			MetaQueryResult->SetMember( "__index", MembersQueryResult );

		}

	}

	// create a new global table
	Lua()->NewGlobalTable( "odbc" );
	ILuaObject* odbc = Lua()->GetGlobal( "odbc" );
	if( odbc )
	{

		// methods
		odbc->SetMember( "NewDatabaseConnection", NewDatabaseConnection );

	}

	return 0;

}

/*------------------------------------
	Shutdown
------------------------------------*/
int Shutdown( lua_State* L )
{

	return 0;

}
